# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

"""Inherit account_invoice to add header discount and early payment discount"""

from osv import osv, fields
from tools.translate import _
import decimal_precision as dp

def intersect(la, lb):
    """returns True for equal keys in two lists"""
    l = filter(lambda x: x in lb, la)
    return len(l) == len(la) == len(lb)

class account_invoice(osv.osv):
    """Inherit account_invoice to add header discount and early payment discount"""

    _inherit = "account.invoice"

    def _amount_all(self, cr, uid, ids, name, args, context=None):
        res = {}
        for invoice in self.browse(cr,uid,ids, context=context):
            f = f = 1 - ((invoice.header_discount + invoice.early_payment_discount or 0) / 100)
            res[invoice.id] = {
                'amount_untaxed': 0.0,
                'amount_tax': 0.0,
                'amount_total': 0.0
            }
            for line in invoice.invoice_line:
                res[invoice.id]['amount_untaxed'] += line.price_unit * line.quantity * f

            for line in invoice.tax_line:
                res[invoice.id]['amount_tax'] += line.amount

            res[invoice.id]['amount_total'] = res[invoice.id]['amount_tax'] + res[invoice.id]['amount_untaxed']

        return res

    def _get_early_discount_amount(self, cr, uid, ids, field_name, arg, context):
        """obtain early_payment_discount_amount"""
        res = {}
        cur_obj = self.pool.get('res.currency')
        for invoice in self.browse(cr, uid, ids):
            if not invoice.early_payment_discount:
                res[invoice.id] = 0.0
                continue
            base_xa_dto = 0.0
            for line in invoice.invoice_line:
                if line.line_apply_header_discounts():
                    base_xa_dto += line.price_subtotal

            #res[invoice.id] = cur_obj.round(cr, uid, invoice.currency_id, (base_xa_dto  * (invoice.early_payment_discount or 0.0)/100.0))
            res[invoice.id] = (base_xa_dto  * (invoice.early_payment_discount or 0.0)/100.0)

        return res

    def _get_header_discount_amount(self, cr, uid, ids, field_name, arg, context):
        """obtain header_discount_amount"""
        res = {}
        cur_obj = self.pool.get('res.currency')
        for invoice in self.browse(cr, uid, ids):
            if not invoice.header_discount:
                res[invoice.id] = 0.0
                continue

            base_xa_dto = 0.0
            for line in invoice.invoice_line:
                if line.line_apply_header_discounts():
                    base_xa_dto += line.price_subtotal

            res[invoice.id] = (base_xa_dto  * (invoice.header_discount or 0.0)/100.0)

        return res

    def _get_invoice_line(self, cr, uid, ids, context=None):
        result = {}
        for line in self.pool.get('account.invoice.line').browse(cr, uid, ids, context=context):
            result[line.invoice_id.id] = True
        return result.keys()

    def _get_invoice_tax(self, cr, uid, ids, context=None):
        result = {}
        for tax in self.pool.get('account.invoice.tax').browse(cr, uid, ids, context=context):
            result[tax.invoice_id.id] = True
        return result.keys()

    _columns = {
        'header_discount': fields.float('Header disc.(%)', digits=(16,2), help="Header discount"),
        'header_discount_amount': fields.function(_get_header_discount_amount, method=True, string="H.D. amount", type='float', digits_compute=dp.get_precision('Account'), help="Header discount amount to apply.", readonly=True),
        'early_payment_discount': fields.float('E.P. disc.(%)', digits=(16,2), help="Early payment discount"),
        'early_discount_amount': fields.function(_get_early_discount_amount, method=True, string="E.P. amount", type='float', digits_compute=dp.get_precision('Account'), help="Early payment discount amount to apply.", readonly=True),
        'amount_untaxed': fields.function(_amount_all, method=True, digits_compute=dp.get_precision('Account'), string='Untaxed',
            store={
                'account.invoice': (lambda self, cr, uid, ids, c={}: ids, ['invoice_line'], 20),
                'account.invoice.tax': (_get_invoice_tax, None, 20),
                'account.invoice.line': (_get_invoice_line, ['price_unit','invoice_line_tax_id','quantity','discount','invoice_id'], 20),
            },
            multi='all'),
        'amount_tax': fields.function(_amount_all, method=True, digits_compute=dp.get_precision('Account'), string='Tax',
            store={
                'account.invoice': (lambda self, cr, uid, ids, c={}: ids, ['invoice_line'], 20),
                'account.invoice.tax': (_get_invoice_tax, None, 20),
                'account.invoice.line': (_get_invoice_line, ['price_unit','invoice_line_tax_id','quantity','discount','invoice_id'], 20),
            },
            multi='all'),
        'amount_total': fields.function(_amount_all, method=True, digits_compute=dp.get_precision('Account'), string='Total',
            store={
                'account.invoice': (lambda self, cr, uid, ids, c={}: ids, ['invoice_line'], 20),
                'account.invoice.tax': (_get_invoice_tax, None, 20),
                'account.invoice.line': (_get_invoice_line, ['price_unit','invoice_line_tax_id','quantity','discount','invoice_id'], 20),
            },
            multi='all'),
    }

    def onchange_partner_id(self, cr, uid, ids, type, partner_id, date_invoice=False, payment_term=False, partner_bank_id=False, company_id=False):
        res = super(account_invoice, self).onchange_partner_id(cr, uid, ids, type, partner_id, date_invoice=date_invoice, payment_term=payment_term, partner_bank_id=partner_bank_id, company_id=company_id)
        if not partner_id:
            res['value']['early_payment_discount'] = False
            res['value']['header_discount'] = False
            return res
        else:
            partner = self.pool.get('res.partner').browse(cr, uid, partner_id, context=None)
            res['value']['header_discount'] = partner.header_discount
            res['value']['early_payment_discount'] = False
            if partner.apply_early_discount:
                if res['value'].get('payment_term', False):
                    pt_obj = self.pool.get('account.payment.term')
                    pt = pt_obj.browse(cr, uid, res['value']['payment_term'], context=None)
                    if pt and pt.apply_early_discount:
                        if partner.early_payment_discount > 0:
                            res['value']['early_payment_discount'] = partner.early_payment_discount
                        else:
                            res['value']['early_payment_discount'] = pt.early_payment_discount

        return res

    def onchange_payment_term(self, cr, uid, ids, payment_term, partner_id, context=None):
        """onchange event to update early payment dicount when the payment term changes"""
        res = {}
        if not payment_term or not partner_id:
            res['early_payment_discount'] = False
            return {'value': res}
        else:
            partner = self.pool.get('res.partner').browse(cr, uid, partner_id, context=None)
            payment_term_obj = self.pool.get('account.payment.term').browse(cr, uid, payment_term, context=None)
            if partner.apply_early_discount and payment_term_obj.apply_early_discount:
                if partner.early_payment_discount > 0:
                    res['early_payment_discount'] = partner.early_payment_discount
                else:
                    res['early_payment_discount'] = payment_term_obj.early_payment_discount
        return {'value': res}

    def compute_invoice_totals(self, cr, uid, inv, company_currency, ref, invoice_move_lines):
        total, total_currency, iml = super(account_invoice,self).compute_invoice_totals(cr, uid, inv=inv, company_currency=company_currency, ref=ref, invoice_move_lines=invoice_move_lines)
        cur_obj = self.pool.get('res.currency')
        if inv.early_payment_discount:
            if inv.journal_id.default_early_payment_discount_account_id:
                diff_currency_p = inv.currency_id.id <> company_currency
                amount_discount = cur_obj.round(cr, uid, inv.currency_id, inv.early_discount_amount)
                if inv.currency_id.id != company_currency:
                    amount_currency = cur_obj.compute(cr, uid,
                            company_currency, inv.currency_id.id,  amount_discount)
                else:
                    amount_currency = False
                iml.append({
                    'type': 'src',
                    'name': "Descuento por pronto pago",
                    'price': amount_discount,
                    'account_id': inv.journal_id.default_early_payment_discount_account_id.id,
                    'date_maturity': inv.date_invoice,
                    'amount_currency': diff_currency_p \
                            and  amount_currency or False,
                    'currency_id': diff_currency_p \
                            and inv.currency_id.id or False,
                    'ref': ref,
                })
                total -= amount_discount

            else:
                raise osv.except_osv(_('Error, falta cuenta para Descuento por pronto pago !'),
                    _('Debe indicar en el diario una cuenta para el descuento por pronto pago.'))
        return total, total_currency, iml

account_invoice()


